/**
 * @author Toru Nagashima
 * @copyright 2015 Toru Nagashima. All rights reserved.
 * See LICENSE file in root directory for full license.
 */
"use strict";

var _getIterator = require("babel-runtime/core-js/get-iterator")["default"];

var _interopRequireDefault = require("babel-runtime/helpers/interop-require-default")["default"];

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports["default"] = spawn;

var _child_process = require("child_process");

var _child_process2 = _interopRequireDefault(_child_process);

// List of child processes that are running currently.
var children = [];

/**
 * Removes this process from the children pool.
 * @this ChildProcess
 */
function removeFromPool() {
    var index = children.indexOf(this);
    if (index !== -1) {
        children.splice(index, 1);
    }
}

/**
 * Kills this process and sub processes with the process group ID.
 * @this ChildProcess
 */
function kill() {
    try {
        process.kill(-this.pid);
    } catch (err) {
        // ignore.
    }
}

/**
 * Launches a new process with the given command.
 * This is almost same as `child_process.spawn`.
 *
 * This detaches the new process to make new process group.
 * And if this process exited before the new process exits, this kills the new process.
 *
 * This returns a `ChildProcess` instance.
 * `kill` method of the instance kills the new process and its sub processes with the process group ID.
 *
 * @param {string} command - The command to run.
 * @param {string[]} args - List of string arguments.
 * @param {object} options - Options.
 * @returns {ChildProcess} A ChildProcess instance of new process.
 * @private
 */

function spawn(command, args, options) {
    options.detached = true; // eslint-disable-line no-param-reassign

    var child = _child_process2["default"].spawn(command, args, options);
    child.on("exit", removeFromPool);
    child.on("error", removeFromPool);
    child.kill = kill;

    // Add to the pool to kill on exit.
    children.push(child);

    return child;
}

// Kill all child processes on exit.
process.on("exit", function () {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = _getIterator(children), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var child = _step.value;

            child.kill();
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator["return"]) {
                _iterator["return"]();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }
});
module.exports = exports["default"];